# iab-vast-model

[![npm](https://img.shields.io/npm/v/iab-vast-model.svg)](https://www.npmjs.com/package/iab-vast-model) [![Dependencies](https://img.shields.io/david/zentrick/iab-vast-model.svg)](https://david-dm.org/zentrick/iab-vast-model) [![Build Status](https://img.shields.io/circleci/project/github/zentrick/iab-vast-model/master.svg)](https://circleci.com/gh/zentrick/iab-vast-model) [![Coverage Status](https://img.shields.io/coveralls/zentrick/iab-vast-model/master.svg)](https://coveralls.io/r/zentrick/iab-vast-model) [![JavaScript Standard Style](https://img.shields.io/badge/code%20style-standard-brightgreen.svg)](http://standardjs.com/)

Base classes for IAB VAST handling. Supports the full IAB VAST 2.0, 3.0, and 4.0
specifications.

To parse a tag into model objects, use
[iab-vast-parser](https://www.npmjs.com/package/iab-vast-parser).
To load and parse a chain of VAST files, use
[iab-vast-loader](https://www.npmjs.com/package/iab-vast-loader).

## Usage

```javascript
import { VAST } from 'iab-vast-model'

const vast = new VAST()
// ... Do your thing ...
```

## Rationale

The API attempts to favor developer experience over mapping the VAST
specifications to classes one-to-one. That has a few implications:

-   You won't find classes corresponding to `<Creatives>`, `<CreativeExtensions>`,
    `<Extensions>`, `<MediaFiles>` and `<Verifications>`. Instead, those have
    been folded into their parent objects.

-   Where a child object can occur multiple times, the corresponding property will
    either be an array or a `SortedList`. The latter models VAST's `sequence`
    attribute and automatically orders elements by its value.

-   Elements that are not required are `null` by default. A typical example is
    `TrackingEvents` on the ads in a `Creative`.

## API

<!-- Generated by documentation.js. Update this documentation by updating the source code. -->

#### Table of Contents

-   [AbsoluteTimeOffset](#absolutetimeoffset)
-   [AbstractAd](#abstractad)
    -   [adSystem](#adsystem)
    -   [conditionalAd](#conditionalad)
    -   [creatives](#creatives)
    -   [errors](#errors)
    -   [extensions](#extensions)
    -   [id](#id)
    -   [impressions](#impressions)
    -   [pricing](#pricing)
    -   [verifications](#verifications)
    -   [viewableImpression](#viewableimpression)
-   [AbstractClicks](#abstractclicks)
    -   [clickThrough](#clickthrough)
    -   [clickTrackings](#clicktrackings)
-   [AbstractExtension](#abstractextension)
    -   [type](#type)
    -   [xmlElement](#xmlelement)
-   [AbstractResource](#abstractresource)
    -   [uri](#uri)
-   [AbstractTimeOffset](#abstracttimeoffset)
    -   [value](#value)
-   [AbstractVerificationResource](#abstractverificationresource)
    -   [apiFramework](#apiframework)
-   [AdBuffet](#adbuffet)
    -   [adPod](#adpod)
    -   [ads](#ads)
-   [SortedList](#sortedlist)
    -   [add](#add)
    -   [clear](#clear)
    -   [get](#get)
    -   [length](#length)
    -   [remove](#remove)
    -   [toArray](#toarray)
-   [TrackingEvents](#trackingevents)
    -   [add](#add-1)
    -   [get](#get-1)
    -   [types](#types)
-   [Creative](#creative)
    -   [adID](#adid)
    -   [apiFramework](#apiframework-1)
    -   [companionAds](#companionads)
    -   [extensions](#extensions-1)
    -   [id](#id-1)
    -   [linear](#linear)
    -   [nonLinearAds](#nonlinearads)
    -   [universalAdId](#universaladid)
-   [Companion](#companion)
    -   [adParameters](#adparameters)
    -   [adSlotID](#adslotid)
    -   [altText](#alttext)
    -   [apiFramework](#apiframework-2)
    -   [assetHeight](#assetheight)
    -   [assetWidth](#assetwidth)
    -   [clickThrough](#clickthrough-1)
    -   [clickTrackings](#clicktrackings-1)
    -   [expandedHeight](#expandedheight)
    -   [expandedWidth](#expandedwidth)
    -   [height](#height)
    -   [id](#id-2)
    -   [pxratio](#pxratio)
    -   [resource](#resource)
    -   [trackingEvents](#trackingevents-1)
    -   [width](#width)
-   [NonLinear](#nonlinear)
    -   [adParameters](#adparameters-1)
    -   [apiFramework](#apiframework-3)
    -   [clickThrough](#clickthrough-2)
    -   [clickTrackings](#clicktrackings-2)
    -   [expandedHeight](#expandedheight-1)
    -   [expandedWidth](#expandedwidth-1)
    -   [height](#height-1)
    -   [id](#id-3)
    -   [maintainAspectRatio](#maintainaspectratio)
    -   [minSuggestedDuration](#minsuggestedduration)
    -   [resource](#resource-1)
    -   [scalable](#scalable)
    -   [width](#width-1)
-   [Linear](#linear-1)
    -   [adParameters](#adparameters-2)
    -   [duration](#duration)
    -   [icons](#icons)
    -   [interactiveCreativeFiles](#interactivecreativefiles)
    -   [mediaFiles](#mediafiles)
    -   [mezzanine](#mezzanine)
    -   [skipoffset](#skipoffset)
    -   [trackingEvents](#trackingevents-2)
    -   [videoClicks](#videoclicks)
-   [AdPod](#adpod-1)
    -   [ads](#ads-1)
-   [AdSystem](#adsystem-1)
    -   [name](#name)
    -   [version](#version)
-   [InLine](#inline)
    -   [adTitle](#adtitle)
    -   [advertiser](#advertiser)
    -   [categories](#categories)
    -   [description](#description)
    -   [surveys](#surveys)
-   [Wrapper](#wrapper)
    -   [allowMultipleAds](#allowmultipleads)
    -   [fallbackOnNoAd](#fallbackonnoad)
    -   [followAdditionalWrappers](#followadditionalwrappers)
    -   [vastAdTagURI](#vastadtaguri)
-   [Icon](#icon)
    -   [apiFramework](#apiframework-4)
    -   [clicks](#clicks)
    -   [duration](#duration-1)
    -   [height](#height-2)
    -   [offset](#offset)
    -   [program](#program)
    -   [pxratio](#pxratio-1)
    -   [resource](#resource-2)
    -   [viewTrackings](#viewtrackings)
    -   [width](#width-2)
    -   [xPosition](#xposition)
    -   [yPosition](#yposition)
-   [MediaFile](#mediafile)
    -   [apiFramework](#apiframework-5)
    -   [bitrate](#bitrate)
    -   [codec](#codec)
    -   [delivery](#delivery)
    -   [height](#height-3)
    -   [id](#id-4)
    -   [maintainAspectRatio](#maintainaspectratio-1)
    -   [maxBitrate](#maxbitrate)
    -   [minBitrate](#minbitrate)
    -   [scalable](#scalable-1)
    -   [type](#type-1)
    -   [uri](#uri-1)
    -   [width](#width-3)
-   [InteractiveCreativeFile](#interactivecreativefile)
    -   [apiFramework](#apiframework-6)
    -   [type](#type-2)
    -   [uri](#uri-2)
-   [Category](#category)
    -   [authority](#authority)
    -   [code](#code)
-   [Click](#click)
    -   [id](#id-5)
    -   [uri](#uri-3)
-   [CompanionAds](#companionads-1)
    -   [companions](#companions)
    -   [required](#required)
-   [CreativeExtension](#creativeextension)
-   [UniversalAdId](#universaladid-1)
    -   [creativeIdentifier](#creativeidentifier)
    -   [idRegistry](#idregistry)
    -   [idValue](#idvalue)
-   [StaticResource](#staticresource)
    -   [creativeType](#creativetype)
-   [Pricing](#pricing-1)
    -   [currency](#currency)
    -   [model](#model)
    -   [value](#value-1)
-   [VideoClicks](#videoclicks-1)
    -   [customClicks](#customclicks)
-   [VAST](#vast)
    -   [errors](#errors-1)
    -   [uri](#uri-4)
    -   [version](#version-1)
-   [Extension](#extension)
-   [FlashResource](#flashresource)
-   [Verification](#verification)
    -   [flashResources](#flashresources)
    -   [javaScriptResources](#javascriptresources)
    -   [vendor](#vendor)
    -   [viewableImpression](#viewableimpression-1)
-   [HTMLResource](#htmlresource)
-   [IconClicks](#iconclicks)
-   [ViewableImpression](#viewableimpression-2)
    -   [id](#id-6)
    -   [notViewables](#notviewables)
    -   [viewables](#viewables)
    -   [viewUndetermineds](#viewundetermineds)
-   [Impression](#impression)
    -   [id](#id-7)
    -   [uri](#uri-5)
-   [IFrameResource](#iframeresource)
-   [JavaScriptResource](#javascriptresource)
-   [NonLinearAds](#nonlinearads-1)
    -   [nonLinears](#nonlinears)
    -   [trackingEvents](#trackingevents-3)
-   [TrackingEvent](#trackingevent)
    -   [offset](#offset-1)
    -   [uri](#uri-6)
-   [RelativeTimeOffset](#relativetimeoffset)
-   [SortedListItem](#sortedlistitem)
    -   [sequence](#sequence)
-   [Survey](#survey)
    -   [type](#type-3)
    -   [uri](#uri-7)

### AbsoluteTimeOffset

**Extends AbstractTimeOffset**

Represents a time offset expressed as an absolute duration in seconds.

### AbstractAd

**Extends SortedListItem**

Abstract class for ads (i.e., [InLine](#inline) and [Wrapper](#wrapper)).

#### adSystem

The ad system for this ad.

Type: [AdSystem](#adsystem)

#### conditionalAd

The conditional attribute for this ad.

Type: [boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)

#### creatives

The creatives for this ad.

Type: [SortedList](#sortedlist)&lt;[Creative](#creative)>

#### errors

The error-tracking URIs for this ad.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)>

#### extensions

The extensions for this ad.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[Extension](#extension)>

#### id

The ad server-defined identifier for this ad.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### impressions

The impression-tracking configurations for this ad.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[Impression](#impression)>

#### pricing

The pricing configuration for this ad.

Type: [Pricing](#pricing)

#### verifications

The verifications for this ad.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[Verification](#verification)>

#### viewableImpression

The viewable impression for this ad.

Type: [ViewableImpression](#viewableimpression)

### AbstractClicks

Stores the click-through URI and a set of click-tracking pixels.

#### clickThrough

The click-through configuration.

Type: [Click](#click)

#### clickTrackings

The click-tracking configurations.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[Click](#click)>

### AbstractExtension

Base class for VAST [Extension](#extension) as well as [CreativeExtension](#creativeextension).

#### type

The MIME type of any code that might be included in the extension.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### xmlElement

The `<Extension>` XML DOM element.

Type: [Element](https://developer.mozilla.org/docs/Web/API/Element)

### AbstractResource

Base class for a verification resource used by a [HTMLResource](#htmlresource), a
[IFrameResource](#iframeresource) and a [StaticResource](#staticresource).

#### uri

The URI to this resource.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

### AbstractTimeOffset

Represents a time offset.

#### value

The value for this offset.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

### AbstractVerificationResource

**Extends AbstractResource**

Base class for a verification resource used by a [FlashResource](#flashresource) and a
[JavaScriptResource](#javascriptresource).

#### apiFramework

The name of the API framework.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

### AdBuffet

Represents a VAST ad buffet.

#### adPod

The ad pod for this ad buffet.

Type: [AdPod](#adpod)

#### ads

The ads in this ad buffet.

Type: [SortedList](#sortedlist)

### SortedList

Represents a list of [SortedListItem](#sortedlistitem)s, ordered by
[SortedListItem#sequence](#sortedlistitemsequence).

#### add

Adds the given item to this list.

**Parameters**

-   `item` **[SortedListItem](#sortedlistitem)** the item.

#### clear

Empties this list.

#### get

Gets the item at the given index.

**Parameters**

-   `index` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** the index.

Returns **[SortedListItem](#sortedlistitem)** the item.

#### length

The length of this list.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

#### remove

Removes the given item from this list.

**Parameters**

-   `item` **[SortedListItem](#sortedlistitem)** the item.

#### toArray

Creates an array representation of this list.

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[SortedListItem](#sortedlistitem)>** the item array.

### TrackingEvents

Configures tracking events. Maps VAST event types to arrays of
[TrackingEvent](#trackingevent) instances.

#### add

Adds a tracking-event configuration for the given event type.

**Parameters**

-   `event` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** the event type.
-   `config` **[TrackingEvent](#trackingevent)** the tracking event configuration.

#### get

Gets the tracking-event configurations for the given event type.

**Parameters**

-   `event` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** the event type.

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[TrackingEvent](#trackingevent)>** the tracking event configurations.

#### types

The event types tracked by this configuration.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)>

### Creative

**Extends SortedListItem**

Represents a creative.

#### adID

The ID of the [Ad](Ad) with which this creative is served.

Required for VAST 2.0 and 3.0, optional for VAST 4.0.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### apiFramework

The string that identifies an API that is needed to execute this creative.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### companionAds

The companion ads for this creative, if any.

Type: [CompanionAds](#companionads)

#### extensions

The creative extensions for this creative.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[CreativeExtension](#creativeextension)>

#### id

The ID of this creative.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### linear

The linear ad for this creative, if any.

Type: [Linear](#linear)

#### nonLinearAds

The non-linear ads for this creative, if any.

Type: [NonLinearAds](#nonlinearads)

#### universalAdId

The unique creative identifier for this creative.

Type: [UniversalAdId](#universaladid)

### Companion

Represents a single companion inside [CompanionAds](#companionads).

#### adParameters

The ad parameters for this companion ad.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### adSlotID

The ad slot ID of this companion ad.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### altText

The alt text of this companion ad.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### apiFramework

The API framework used by this companion ad.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### assetHeight

The height of this companion ad's asset in pixels.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

#### assetWidth

The width of this companion ad's asset in pixels.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

#### clickThrough

The click-through configuration.

Type: [Click](#click)

#### clickTrackings

The click-tracking configurations.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[Click](#click)>

#### expandedHeight

The expanded height of this companion ad in pixels.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

#### expandedWidth

The expanded width of this companion ad in pixels.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

#### height

The height of this companion ad in pixels.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

#### id

The ID for this companion ad.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### pxratio

The pixel ratio for which the companion creative is intended.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### resource

The resource associated with this companion ad.

Type: ([StaticResource](#staticresource) \| [IFrameResource](#iframeresource) \| [HTMLResource](#htmlresource))

#### trackingEvents

The event-tracking configuration for this companion ad.

Type: [TrackingEvents](#trackingevents)

#### width

The width of this companion ad in pixels.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

### NonLinear

Represents a single non-linear ad inside [NonLinearAds](#nonlinearads).

#### adParameters

The ad parameters for this non-linear ad.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### apiFramework

The API framework used by this non-linear ad.

VAST 2.0 and 3.0 only.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### clickThrough

The click-through configuration for this non-linear ad.

Type: [Click](#click)

#### clickTrackings

The click-tracking configurations for this non-linear ad.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[Click](#click)>

#### expandedHeight

The expanded height of this non-linear ad in pixels.

VAST 2.0 and 3.0 only.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

#### expandedWidth

The expanded width of this non-linear ad in pixels.

VAST 2.0 and 3.0 only.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

#### height

The height of this non-linear ad in pixels.

VAST 2.0 and 3.0 only.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

#### id

The ID of this non-linear ad.

VAST 2.0 and 3.0 only.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### maintainAspectRatio

Whether this non-linear ad's aspect ratio should be maintained when scaled.

VAST 2.0 and 3.0 only.

Type: [boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)

#### minSuggestedDuration

The minimum suggested duration of this non-linear ad in seconds.

VAST 2.0 and 3.0 only.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

#### resource

The resource associated with this non-linear ad.

Type: Resource

#### scalable

Whether this non-linear ad is meant to scale to larger dimensions.

VAST 2.0 and 3.0 only.

Type: [boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)

#### width

The width of this non-linear ad in pixels.

VAST 2.0 and 3.0 only.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

### Linear

Represents a linear within a [Creative](#creative).

#### adParameters

The ad parameters for this linear creative.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### duration

The duration of this linear creative in seconds.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

#### icons

The icons for this linear creative.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[Icon](#icon)>

#### interactiveCreativeFiles

The interactive creative files for this linear creative.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[InteractiveCreativeFile](#interactivecreativefile)>

#### mediaFiles

The media files for this linear creative.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[MediaFile](#mediafile)>

#### mezzanine

The URI of the raw mezzanine file for this linear creative, intended for
video transcoding.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### skipoffset

The time interval after which this linear creative can be skipped.

Type: TimeOffset

#### trackingEvents

The event-tracking configuration for this linear creative.

Type: [TrackingEvents](#trackingevents)

#### videoClicks

The video click tracking configuration for this linear creative.

Type: [VideoClicks](#videoclicks)

### AdPod

Represents a VAST ad pod.

#### ads

The ads in this ad pod.

Type: [SortedList](#sortedlist)

### AdSystem

Stores information about the ad system.

#### name

The name of the ad server that returned the ad.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### version

The version of the ad system.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

### InLine

**Extends AbstractAd**

Represents an InLine ad.

#### adTitle

The title for this ad.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### advertiser

The advertiser name for this ad.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### categories

The categories for this ad.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[Category](#category)>

#### description

The description for this ad.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### surveys

The survey URI for this ad.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[Survey](#survey)>

### Wrapper

**Extends AbstractAd**

Represents a wrapper ad.

#### allowMultipleAds

Whether multiple ads are allowed in the requested VAST response.

Type: [boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)

#### fallbackOnNoAd

Whether to use an available Ad when the requested VAST response returns no
ads.

Type: [boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)

#### followAdditionalWrappers

Whether subsequent wrappers after a requested VAST response is allowed.

Type: [boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)

#### vastAdTagURI

The URI to a VAST response that may be another VAST [Wrapper](#wrapper) or a
VAST [InLine](#inline) ad.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

### Icon

Represents an icon used in a [Linear](#linear) creative.

#### apiFramework

The API framework used by this icon.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### clicks

The click-tracking configuration for this icon.

Type: [IconClicks](#iconclicks)

#### duration

The duration of this icon in seconds.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

#### height

The height of this icon in pixels.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

#### offset

The time offset of this icon in seconds.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

#### program

The industry initiative that this icon supports.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### pxratio

The pixel ratio for which the icon creative is intended.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### resource

The resource associated with this icon.

Type: Resource

#### viewTrackings

The view-tracking URIs for this icon.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)>

#### width

The width of this icon in pixels.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

#### xPosition

The horizontal position of this icon. Either a number of pixels or the
string `"left"` or `"right"`.

Type: ([number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number) \| [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String))

#### yPosition

The vertical position of this icon. Either a number of pixels or the
string `"top"` or `"bottom"`.

Type: ([number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number) \| [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String))

### MediaFile

Represents a media file used in a [Linear](#linear) creative.

#### apiFramework

The API framework used by this media file.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### bitrate

The bitrate of this media file in kbps.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

#### codec

The codec of this media file in RFC 4281 format.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### delivery

The delivery method for this media file.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### height

The height of this media file in pixels.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

#### id

The ID of this media file.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### maintainAspectRatio

Whether this media file's aspect ratio should be maintained when scaled.

Type: [boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)

#### maxBitrate

The maximum bitrate of this media file in kbps.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

#### minBitrate

The minimum bitrate of this media file in kbps.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

#### scalable

Whether this media file is meant to scale to larger dimensions.

Type: [boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)

#### type

The MIME type of this media file.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### uri

The URI to this media file.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### width

The width of this media file in pixels.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

### InteractiveCreativeFile

Represents an interactive creative file used in a [Linear](#linear) creative.

#### apiFramework

The API framework used by this interactive creative file.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### type

The MIME type of this interactive creative file.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### uri

The URI to this interactive creative file.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

### Category

Pricing information for real-time bidding.

#### authority

The URL for the organizational authority that produced the list being used
to identify the ad content.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### code

The category code or label that identifies the ad content.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

### Click

Describes a click configuration.

#### id

The ID of this click.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### uri

The URI of this tracking pixel.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

### CompanionAds

Represents companion ads within a [Creative](#creative).

#### companions

The companion ads.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[Companion](#companion)>

#### required

Determines which companion creative to display when multiple companions
are supplied and whether the ad can be displayed without its companion
creative. Either `"all"`, `"any"`, or `"none"`.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

### CreativeExtension

**Extends AbstractExtension**

Represents a creative extension.

### UniversalAdId

Represents a unique identifier for an ad.

#### creativeIdentifier

The string identifying the unique creative identifier.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### idRegistry

The registry URL for the unique creative identifier.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### idValue

A string for the unique creative identifier.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

### StaticResource

**Extends AbstractResource**

Describes non-HTML creative.

#### creativeType

The MIME type of this resource.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

### Pricing

Represents pricing information for real-time bidding.

#### currency

The three-letter ISO-4217 currency symbol that identifies the currency of
the value provided (e.g. `"USD"`, `"GBP"`, etc.).

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### model

The pricing model. Either `"CPM"`, `"CPC"`, `"CPE"`, or `"CPV"`.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### value

The price that can be used in real-time bidding systems.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

### VideoClicks

**Extends AbstractClicks**

Represents the click-tracking configuration for a [Linear](#linear) creative.

#### customClicks

The custom-click configurations.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[Click](#click)>

### VAST

**Extends AdBuffet**

Represents a VAST document.

#### errors

The error-tracking URIs for this document.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)>

#### uri

The URI associated with this document.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### version

The VAST version used by this document.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

### Extension

**Extends AbstractExtension**

Represents a VAST extension.

### FlashResource

**Extends AbstractVerificationResource**

Describes a Flash resource used to collect verification data.

### Verification

Represents a list of verification resources for a vendor.

#### flashResources

The Flash resources for this verification vendor.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[FlashResource](#flashresource)>

#### javaScriptResources

The JavaScript resources for this verification vendor.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[JavaScriptResource](#javascriptresource)>

#### vendor

The home page URL for the verification service provider that supplies the
resource file.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### viewableImpression

The viewable impression for this verification vendor.

Type: [ViewableImpression](#viewableimpression)

### HTMLResource

**Extends AbstractResource**

Describes an HTML snippet.

### IconClicks

**Extends AbstractClicks**

Represents the click tracking configuration for an [Icon](#icon).

### ViewableImpression

Represents the impression-tracking URIs for both an [InLine](#inline) and any
[Wrapper](#wrapper).

#### id

The ad server ID of this viewability impression pixel.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### notViewables

The URIs that direct the video player to a tracking resource file that the
video player should request if the ad is executed but never meets criteria
for a viewable impression.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)>

#### viewables

The URIs that direct the video player to a tracking resource file that the
video player should request at the time that criteria is met for a viewable
impression.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)>

#### viewUndetermineds

The URIs that direct the video player to a tracking resource file that the
video player should request if the player cannot determine whether criteria
is met for a viewable impression.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)>

### Impression

Represents an impression.

#### id

The ad server id for the impression.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### uri

The URI of this impression resource.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

### IFrameResource

**Extends AbstractResource**

Describes an HTML page for display within an iframe.

### JavaScriptResource

**Extends AbstractVerificationResource**

Describes a JavaScript resource used to collect verification data.

### NonLinearAds

Represents non-linear ads within a [Creative](#creative).

#### nonLinears

The non-linear ads.

Type: [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[NonLinear](#nonlinear)>

#### trackingEvents

The event-tracking configuration.

Type: [TrackingEvents](#trackingevents)

### TrackingEvent

Represents an event to be tracked.

#### offset

The time offset for this event configuration.

Type: TimeOffset

#### uri

The tracking URI for this event configuration.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

### RelativeTimeOffset

**Extends AbstractTimeOffset**

Represents a time offset expressed as a percentage (between 0 and 100).

### SortedListItem

Represents an item in a [SortedList](#sortedlist). Base class for [Ad](Ad) and
[Creative](#creative).

#### sequence

The sequence number of this item within its parent.

Type: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)

### Survey

Represents survey information for data collection.

#### type

The MIME type of the resource being served.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

#### uri

The URI to any resource relating to an integrated survey.

Type: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)

## TODO

The main thing that's missing is **validation**. There are a few steps to this:

-   Use [validator](https://www.npmjs.com/package/validator) to check all values
    passed to setters.
-   Verify that every required property is set.
-   Write tests for the validation code.
-   Enclose all validation code in marker comments (e.g., `// <strict>`). This
    will allow us to strip it out and produce a lightweight "loose" build without
    any validation, making tags load faster.

PRs welcome!

## Maintainers

-   [Tim De Pauw](https://github.com/timdp)
-   [Christophe Bonello](https://github.com/cbonello)

## License

MIT
